#version 150
precision highp float; precision highp int;
//Ethan Alexander Shulman 2024 "Rivers of Time" October 26, 2024

in vec3 in_Position;
out vec4 uv;
out float time;

layout(std140) uniform ShaderviewBlock {
	vec4 mouse;
	float screenX,screenY;
	int frame,framerate;
};

#define ANIMATION 1
#ifdef ANIMATION
#define AA_SAMPLES 32
#else
#define AA_SAMPLES 1000000
#endif


#define PI 3.141592653589793


mat2 r2d(float a) {
	float c = cos(a), s = sin(a);
	return mat2(c,s,-s,c);
}
//r2 hash http://extremelearning.com.au/unreasonable-effectiveness-of-quasirandom-sequences/
vec2 quasiRandom(float n) {
	#define R2V 1.32471795724474602596
	return fract(.5+n*(1./vec2(R2V,R2V*R2V)));
}


float twave(float x) {
	return abs(fract(x)-.5)*2.;
}



uniform sampler2D Buffer0;

void vert() {
	#ifdef ANIMATION
	time = min(60., float(frame)/float(framerate*AA_SAMPLES));
	#else
	//screenshot 0
	time = 5.;
	
	//screenshot 1
	//time = 15.;
	
	//screenshot 2
	//time = 30.;
	#endif
	
	uv = vec4((in_Position.xy*vec2(screenX/screenY,1)+(quasiRandom(float(frame%AA_SAMPLES))-.5)*2./screenY),
		(in_Position.xy*.5+.5)*vec2(screenX,screenY));
	gl_Position = vec4(in_Position,1);
}
void frag() {
	vec2 p = uv.xy;

	for (int i = 0; i < 4; i++) {
		float a = atan(p.x,p.y);
		p -= p.yx/(1e-3+twave(1./length(p)))*twave((a/PI+1.)+time/60.);
	}
	

	vec3 c = pow(vec3(twave((atan(p.x,p.y)/PI+1.)*2.)), vec3(1.2, 1.05, .95));

	vec4 sum = texelFetch(Buffer0,ivec2(uv.zw),0);
	if (frame%AA_SAMPLES == 0) sum = vec4(0);
	sum += vec4(c, 1.);
	gl_FragColor = sum;
}
